package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.*;
import com.ibm.dse.automaton.*;
import com.ibm.dse.cs.html.HtmlConstants ;
/**
 * This class represents a state for an HTML processor.
 *
 * @copyright(c) Copyright IBM Corporation 2000
 */
public class DSEHtmlState extends DSEState
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	public static final java.lang.String FINAL_PAGE = "finalPage";
	public static final java.lang.String CONSEQ = "conseq";
	public static final java.lang.String INPUT_MAP_FMT = "inputMapFmt";
	public static final java.lang.String OUTPUT_MAP_FMT = "outputMapFmt";
	private java.lang.String consequentiality = DSEHtmlProcessor.CONSEQ_UNDEFINED;
	private boolean ignoreReturnType = false;
	public com.ibm.dse.base.Tag inputFormatTag = null;
	public com.ibm.dse.base.Tag outputFormatTag = null;
	protected java.lang.String finalPage;
	
	/** <HSBC> **/
	public static final String INTER_STATE_FMT = "interStateFmt";
	public static final String KEEP_CTXT_FOR_VALIDATION = "keepCtxt";
	protected Tag contextTag = null;
	protected Context stateContext = null;
	protected Tag interStateFmt = null;
	protected boolean keepCtxt = false;
	/** </HSBC> **/
	
/**
 * The HtmlState has two special types which differentiates it from the State.
 * Type PAGE: this state returns control to the user, and displays an HTML page.
 * When this page is posted, the state is activated another time. We must ensure
 * that the actions associated are processed only once (the first time the state is
 * activated) and events are processed as well only once (the second time that it
 * is activated).
 * Type SUBFLOW: Same as PAGE, but it runs a subFlow instead of displaying a page.
 * This two types don't receive events from actions or from notifiers in the context
 * hierarchy, the events are received only from the HTML page (type PAGE) or
 * from the typeIdInfo of the child process (type SUBFLOW).
 */

public void activate() throws DSEInvalidArgumentException, DSEProcessorException
{
	if ( getType() == null || ( !getType().equals(SUBFLOW_TYPE) && !getType().equals(PAGE_TYPE) && !getType().equals(FINAL_TYPE) ) )
	{
		// If use an HtmlState as a normal State, call the parent method.
		super.activate();
	}
	
	else if ( getType().equals(SUBFLOW_TYPE) || getType().equals(PAGE_TYPE) )
	{
		// Set the running mode flag.
		runningModeFlag = true;
		
		// First time an special type (SUBFLOW & TYPE) pass through here must only execute the actions.
		if (!ignoreReturnType)
		{
			// Instantiates the entryActions and execute them sequentially.
			if ( (!ignoreEntryActions) && (entryActionsTags != null) )
			{
				doEntryActions( (Vector) null);
			}
		}
		
		// Second time an special type (SUBFLOW & TYPE) pass through here must only process events.
		else
		{
			processEventQueue(null);
		}
	}
		
	// If type is FINAL do not process more events, and ensure control will be returned (ignoreReturnType).
	else if (getType().equals(FINAL_TYPE))
	{
		//Maybe the final state has entry actions that have to be executed.
		runningModeFlag = true;
		
		if ( (!ignoreEntryActions) && (entryActionsTags != null) )
		{
			doEntryActions( (Vector) null);
		}
		
		setIgnoreReturnType(false);
		setIgnoreEntryActions(true);
	}
}
/**
 * This utility method is used to change the state of a processor in
 * response to a change state event. It extracts required information
 * from the event object and uses this information to change state.
 * 
 * @param event com.ibm.dse.base.DSEEventObject
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
protected void changeProcessorState(DSEEventObject event) throws com.ibm.dse.automaton.DSEProcessorException
{
	String nextStateName = (String) event.getParameters().get("nextStateName");

	if (processor instanceof DSEHtmlProcessor)
	{
		String transitionName = (String) event.getParameters().get("transitionName") ;
		if(transitionName != null){
			((DSEHtmlProcessor) processor).changeState(nextStateName, false, transitionName) ;
		}else{
			boolean ignoreFlag = ((Boolean) event.getParameters().get("ignoreEntryActions")).booleanValue();
			((DSEHtmlProcessor) processor).changeState(nextStateName, ignoreFlag) ;
		}
	}
	else
	{
		boolean ignoreFlag = ((Boolean) event.getParameters().get("ignoreEntryActions")).booleanValue();
		processor.changeState(nextStateName, ignoreFlag);
	}
}
/**
 * Excecutes the entry actions for this state.
 * 
 * @param otherTrigs com.ibm.dse.base.Vector
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
protected void doEntryActions(Vector otherTrigs) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.automaton.DSEProcessorException
{
	try
	{
		executeActions(getEntryActions(), null, otherTrigs, false, null) ;
	}
	catch(DSEException dsee)
	{
		throw new DSEProcessorException(DSEException.critical, null, "Exception caught while executing entry actions for state: " + getName() + ": " + dsee.getMessage()) ;
	}
}
/**
 * This utility method executes the actions associated with the supplied transition.
 * If this method completes successfully, it fires a change state event to be handled
 * by the state. It returns a boolean to indicate if any event was fired.
 *
 * @return boolean
 * @param transition com.ibm.dse.automaton.TransitionDescriptor
 * @param sourceEvent com.ibm.dse.base.DSEEventObject
 * @param otherTrigs com.ibm.dse.base.Vector
 * @exception com.ibm.dse.base.DSEException
 */
protected boolean executeActions(TransitionDescriptor transition, DSEEventObject sourceEvent, Vector otherTrigs) throws com.ibm.dse.base.DSEException
{
	String transitionName = null ;

	transitionName = transition.getName() ;
	if(transitionName == null)
	{
		try
		{
			transitionName = sourceEvent.getSourceName() + "." + sourceEvent.getName() ;
		}
		catch(Exception e) {}
	}

	return executeActions(transition.getActionsTags(), sourceEvent, otherTrigs, transition.isIgnoreEntryActions(), transitionName) ;
}
/**
 * This method executes the actions associated with the action tags in the
 * supplied vector of tags. If this method completes successfully, it fires
 * a change state event to be handled by the state. The boolean and String
 * arguments are passed in any change state event fired from this method.
 *
 * @return boolean
 * @param actionTags com.ibm.dse.base.Vector
 * @param sourceEvent com.ibm.dse.base.DSEEventObject
 * @param otherTrigs com.ibm.dse.base.Vector
 * @param ignoreFlag boolean
 * @param transitionName java.lang.String
 * @exception com.ibm.dse.base.DSEException
 */
protected boolean executeActions(Vector actionTags, DSEEventObject sourceEvent, Vector otherTrigs, boolean ignoreFlag, String transitionName) throws com.ibm.dse.base.DSEException
{
	Action action = null ;
	GuardCondition condition = null ;
	java.util.Enumeration e = actionTags.elements() ;
	boolean skipAction = false, handlerAdded = false, contextChained = false ;

	// loop through all the actions for these transitions
	while(runningModeFlag && e.hasMoreElements())
	{
		skipAction = false ;
		handlerAdded = false ;
		action = getInstanceOfAction((Tag) e.nextElement());

		contextChained = ((DSEAction)action).chainOrAssignContext();
		action.setSourceEvent(sourceEvent);
		handlerAdded = addHandlersForAction(action, otherTrigs);

		// check guard conditions for this action
		int result = Integer.MIN_VALUE ;
		java.util.Enumeration conditions = action.getGuardConditions().elements() ;
		while (conditions.hasMoreElements())
		{
			condition = (GuardCondition) conditions.nextElement() ;
			condition.setSourceEvent(sourceEvent) ;
			result = getGuardConditionResult(action, condition);

			if(result == CONTINUE) continue;
			else if(result == SKIP_CONDITIONS) break;
			else if(result == SKIP_ACTION)
			{
				skipAction = true ;
				break ;
			}
			else if(result == SKIP_ALL_ACTIONS) return false ;
			else if (result == ABORT_PROCESS)
			{
				getProcessor().abort() ;
				return false ;
			}
			else if(result == CHANGE_STATE)
			{
				if(transitionName != null){
					fireDSEChangeStateEvent(condition.getNewStateName(), ignoreFlag, transitionName) ;
				}else{
					fireDSEChangeStateEvent(condition.getNewStateName(), ignoreFlag);
				}
				return true ;
			}
			else if(result == EXECUTE_ACTION_AND_CHANGESTATE)
			{
				try
				{
					action.preProcess();
					//trace processor execute the action
	                if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		                Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),this.getName() + " in Processor(" + this.getProcessor().getName() + ") execute the action:" + action.getName());
	               
	
					action.execute();
				}
				catch (Exception exc)
				{
					// if the action throws an exception, remove the handlers for
					// this action and throw a new exception.
					if (handlerAdded) handlerAdded=removeHandlersForAction(action);
					throw new DSEProcessorException(DSEException.critical, Processor.COMPID, exc.toString());
				}

				// clean up
				if (handlerAdded) removeHandlersForAction(action);
				if(contextChained) action.close(contextChained);
				else action.close() ;

				fireDSEChangeStateEvent(condition.getNewStateName(), ignoreFlag, transitionName) ;
				return true ;
			}
		}

		if(skipAction) continue ;

		// once guard conditions have been evaluated, then execute the action
		try
		{
			action.preProcess();
			//trace processor execute the action
	        if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		        Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),this.getName() + " in Processor(" + this.getProcessor().getName() + ") execute the action:" + action.getName());
	        
	
			action.execute();
		}
		catch (Exception exc)
		{
			// if the action throws an exception, remove the handlers for
			// this action and throw a new exception.
			if (handlerAdded) handlerAdded=removeHandlersForAction(action);
			throw new DSEProcessorException(DSEException.critical, Processor.COMPID, exc.toString());
		}

		// clean up
		if (handlerAdded) removeHandlersForAction(action);
		if(contextChained) action.close(contextChained);
	}

	return false ;
}
/**
 * Fires a change state event to the processor.
 *
 * @param targetStateName java.lang.String
 * @param ignoreEntryActionsFlag boolean
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
protected void fireDSEChangeStateEvent(String targetStateName, boolean ignoreEntryActionsFlag) throws com.ibm.dse.automaton.DSEProcessorException
{
	super.fireDSEChangeStateEvent(targetStateName, new Boolean(ignoreEntryActionsFlag), new Boolean(false));
}
/**
 * This method dispatches a change state event with required information
 * 
 * @param targetStateName java.lang.String
 * @param ignoreEntryActionsFlag boolean
 * @param transitionName java.lang.String
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
protected void fireDSEChangeStateEvent(String targetStateName, boolean ignoreEntryActionsFlag, String transitionName) throws com.ibm.dse.automaton.DSEProcessorException
{
	try
	{
		Hashtable param = new Hashtable(4);
		param.put("nextStateName", targetStateName);
		param.put("ignoreEntryActions", new Boolean(ignoreEntryActionsFlag)) ;
		param.put("ignoreExitActions", new Boolean(false)) ;
		param.put("transitionName", transitionName) ;
		DSEEventObject ev = new DSEEventObject("dse_changeState", ((DSEProcessor) getProcessor()), param);
		dispatchEvent(ev);
	}
	catch (DSEInvalidArgumentException ina)
	{
		if (Trace.doTrace(Processor.COMPID, Trace.High, Trace.Debug))
		{
			Trace.trace(Processor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "DSESHtmlState: Could not fire a event for target state named:  " + targetStateName) ;
		}
	}
}
/**
 * This method dispatches a change state event with required information
 * 
 * @param targetStateName java.lang.String
 * @param ignoreEntryActionsFlag boolean
 * @param ignoreExitActionsFlag boolean
 * @param transitionName java.lang.String
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
protected void fireDSEChangeStateEvent(String targetStateName, boolean ignoreEntryActionsFlag, boolean ignoreExitActionsFlag, String transitionName) throws com.ibm.dse.automaton.DSEProcessorException
{
	try
	{
		Hashtable param = new Hashtable(4);
		param.put("nextStateName", targetStateName);
		param.put("ignoreEntryActions", new Boolean(ignoreEntryActionsFlag)) ;
		param.put("ignoreExitActions", new Boolean(ignoreExitActionsFlag)) ;
		param.put("transitionName", transitionName) ;
		DSEEventObject ev = new DSEEventObject("dse_changeState", ((DSEProcessor) getProcessor()), param);
		dispatchEvent(ev);
	}
	catch (DSEInvalidArgumentException ina)
	{
		if (Trace.doTrace(Processor.COMPID, Trace.High, Trace.Debug))
		{
			Trace.trace(Processor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "DSESHtmlState: Could not fire a event for target state named:  " + targetStateName) ;
		}
	}
}
/**
 * This method returns the consequentiality of this state. It will be one of
 * the following values from DSEHTMLProcessor: CONSEQ_UNDEFINED, CONSEQ_TRUE,
 * or CONSEQ_FALSE ;
 * 
 * @return java.lang.String
 */
public java.lang.String getConsequentiality() {
	return consequentiality;
}

/** 
 * <HSBC>
 * Returns the state context
 */
public Context getContext() {
	return stateContext;
}
/** 
 * <HSBC>
 * Returns the context tag
 */
public Tag getContextTag() {
	return contextTag;
}
/** 
 * <HSBC>
 * Returns inter state format
 */
public Tag getInterStateFormatTag() {
	return interStateFmt;
}

/**
 * This method gets the final page of this state. This atribute only makes sense 
 * in final states. Represents the name of the page that will be displayed when
 * the processor reaches the final state.
 *
 * @return java.lang.String
 */
public java.lang.String getFinalPage() {
	return finalPage;
}
/**
 * Returns the input format tag attribute of this class.
 * 
 * @return com.ibm.dse.base.Tag
 */
public com.ibm.dse.base.Tag getInputFormatTag() {
	return inputFormatTag;
}
/**
 * Returns the output format tag attribute of this class.
 * 
 * @return com.ibm.dse.base.Tag
 */
public com.ibm.dse.base.Tag getOutputFormatTag() {
	return outputFormatTag;
}
/**
 * Initializes the state of this object from externalized data
 * 
 * @return java.lang.Object
 * @param aTag com.ibm.dse.base.Tag
 * @exception java.io.IOException
 * @exception com.ibm.dse.base.DSEException
 */
public Object initializeFrom(Tag aTag) throws java.io.IOException, com.ibm.dse.base.DSEException
{
	super.initializeFrom(aTag) ;
	Hashtable h=new Hashtable();
	setIgnoreReturnType(false) ;

	// The default behaviour is for a state not to specify consequentiality
	setConsequentiality(DSEHtmlProcessor.CONSEQ_UNDEFINED) ;

	// get useful attributes
	String name = null, value = null ;
	TagAttribute attribute = null ;

	for(java.util.Enumeration e = aTag.getAttrList().elements(); e.hasMoreElements(); )
	{
		attribute = (TagAttribute) e.nextElement() ;

		name = attribute.getName() ;
		value = (String) attribute.getValue() ;
		if(name.equals(DSEHtmlState.CONSEQ))
		{
			if(value.equals(Constants.True))
			{
				setConsequentiality(DSEHtmlProcessor.CONSEQ_TRUE) ;
			}
			else if(value.equals(Constants.False))
			{
				setConsequentiality(DSEHtmlProcessor.CONSEQ_FALSE) ;
			}
			else throw new DSEException(DSEException.harmless, null, "Undefined value specified for conseq attribute in tag definition: " + value) ;
		}

		if(name.equals(DSEHtmlState.FINAL_PAGE)) setFinalPage(value);

		/** <HSBC> **/
		// Set KEEP_CTXT_FOR_VALIDATION value if defined
		if(name.equals(DSEHtmlState.KEEP_CTXT_FOR_VALIDATION)) {
			if (value.equals(Constants.True)) {
				setKeepContext(true);
			}
		}
		/** <HSBC> **/

	}

	Tag nextSubTag;
	java.util.Enumeration attributes;
	TagAttribute refname=null;
	Tag format=null;
	try
	{
		int nTags = aTag.getSubTags().size();
		for (int i = 0; i<nTags; i++)
		{
			nextSubTag = (Tag)(aTag.getSubTags().elementAt(i));
			if (nextSubTag.getName()!=null)
			{				
				if (nextSubTag.getName().equals(Constants.RefFormat))
				{
					format=nextSubTag.getSubTag(0);//only there is one subtag
					attributes = nextSubTag.getAttrList().elements();
					while (attributes.hasMoreElements())
					{
						attribute = (TagAttribute) attributes.nextElement();
						if (attribute.getName().equals(Constants.Name))
						{
							if (attribute.getValue().equals(DSEHtmlState.INPUT_MAP_FMT))
							{	
								setInputFormatTag(format);
			
							}
							else if (attribute.getValue().equals(DSEHtmlState.OUTPUT_MAP_FMT))
							
							{	
								setOutputFormatTag(format);
							}
							/** <HSBC> **/
							// Set InterStateFmt value if defined
							else if (attribute.getValue().equals(DSEHtmlState.INTER_STATE_FMT))
							{	
								setInterStateFormatTag(format);
							}
							/** </HSBC> **/
						}
					}
				}
				
				/** <HSBC> **/
				// Set StateContext value if defined
				else if (nextSubTag.getName().equals(Constants.Context)) {
					setContext((Context)Context.getExternalizer().convertTagToObject(nextSubTag));
					setContextTag(nextSubTag);
				}
				/** </HSBC> **/

			}//end if null

		}//end for
	}
	catch(Exception e){
		throw new java.io.IOException(e.toString());
	}
	
	return this ;
}
/**
 * Returns a boolean indicating whether normal checking for return type
 * should be ignored for this state. Return types were introduced for HTML
 * processor states, and are used to process certain state types for which
 * we wish to return control from the normal state machine flow. We however
 * may wish to execute these states normally after the first time they return
 * control. The return type attribute controls whether or not a state that
 * would normally return control will ignore it. This method returns the
 * current value of this attribute.
 * 
 * @return boolean
 */
public boolean isIgnoreReturnType() {
	return ignoreReturnType;
}

/**
 *  <HSBC>
 *  Returns a boolean indicate keeping the context for the duration of state 
 *  or not
 */
public boolean isKeepContext() {
	return keepCtxt;
}

/**
 * This method does most of the logical work of the state. It queries the event
 * event queue for an event defining a transition from the state. It then executes
 * any actions corresponding to this transition, and declares interest in changing
 * to a new state by firing a change state event. This event is also trapped in this
 * method and used to change to the new state.
 *
 * The major portion of the logic of this code has been retained from the parent class,
 * DSEState. However, there are significant changes that have been made to support the
 * features required for the HomeBanking for WSSC HTML Client. Make special note that
 * changes to the corresponding method in the superclass might have to be synched up
 * with this method.
 *
 * @note - The event handling for this method is coded in such a way that events are 
 * @note - processed up until a point at which the change state event is obtained and
 * @note - handled. In reality we expect only 2 events to be processed by this method,
 * @note - one event to define what transition is used to go to a new state, and one
 * @note - that  actually triggers the change of state. The caveat of this is that if
 * @note - there is more than one event in the queue at the beginning of this method,
 * @note - or new events are added during processing (e.g. from other threads), then the
 * @note - actions for more than one transition may be executed, and the state machine
 * @note - will also be changed to a state corresponding to the one defined for the last
 * @note - transition processed.
 * @note - Also note that this method will not return until at least one transition has
 * @note - been processed, since a change event is not triggered until a transition is
 * @note - processed. This holds true unless an external source (like an action) triggers
 * @note - a change state event.
 *
 * @param otherTrigs com.ibm.dse.base.Vector
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
protected void processEventQueue(Vector otherTrigs) throws DSEInvalidArgumentException, DSEProcessorException
{
	String trigName = null, notifierName = null, eventName = null ;

	try
	{
		Processor processor = getProcessor() ;

		while(true)
		{
			// get next event from the processor event queue
			DSEEventObject anEvent = null ;
			try
			{
				pqThread = Thread.currentThread();
				anEvent = (DSEEventObject) processor.getEventQueue().get();
				pqThread = null;
			}
			catch (InterruptedException e)
			{
				if(getProcessor().getStatus().equals(DSEHtmlProcessor.ABORTING)) return ;
				else continue ;
			}
			//process the event
			//trace process the event
	        if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		        Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),this.getName()+" in Processor("+this.getProcessor().getName() + ") is processing the event:" + anEvent.getName());
	        
	


			// if the event corresponds to a change state event, then change the
			// processor state and return control to the caller.
			if (anEvent.getName().equals("dse_changeState"))
			{
				// Just before changing the state, execute the exit actions
				boolean ignoreExitActions = ( (Boolean) anEvent.getParameters().get("ignoreExitActions") ).booleanValue();
				if ( !getExitActions().isEmpty() && !ignoreExitActions )
					doExitActions();
			
				changeProcessorState(anEvent) ;
				break ;
			}

			// if event is not a change state event then get a transition and
			// execute its actions.
			else
			{
				// get a transition for this event
				TransitionDescriptor transition = null ;
				transition = (TransitionDescriptor) transitions.get(anEvent.getSourceName() + "." + anEvent.getName());
			
				// if no transition exists for named event, then continue
				if(transition == null)	{
					throw new com.ibm.dse.clientserver.DSECSInvalidRequestException("The automaton has received a non existent event");
				}

				// if  we have a valid transition, then execute its actions
				else
				{
					// execute actions
					boolean eventFired = executeActions(transition, anEvent, otherTrigs) ;

					// check processor status after executing transition actions.
					// The processor may have aborted, in which case we exit this
					// method
					if(processor.getStatus().equals(DSEProcessor.ABORTED)) return ;

					// if executing the actions did not trigger a change state event,
					// then trigger one now
					if((!eventFired)&&(!(transition.getTargetStateName()==null)))
					{
						fireDSEChangeStateEvent(transition.getTargetStateName(), transition.isIgnoreEntryActions(), transition.isIgnoreExitActions(), transition.getName()) ;
					}
				}
			} 
		} 
	} 
	catch (DSEException dsee)
	{
		throw new DSEInvalidArgumentException(DSEException.critical, name, dsee.toString());
	}

	return ;
}
/**
 * This method sets the consequentiality of this state. It should be one of
 * the following values from DSEHTMLProcessor: CONSEQ_UNDEFINED, CONSEQ_TRUE,
 * or CONSEQ_FALSE ;
 * 
 * @param newConsequentiality java.lang.String
 */
public void setConsequentiality(java.lang.String newConsequentiality)
{
	consequentiality = newConsequentiality;
}

/**
 * <HSBC>
 * Set the state context
 */
public void setContext(Context context) {
	stateContext = context;
}
/**
 * <HSBC>
 * Set the context tag
 */
public void setContextTag(Tag aTag) {
	contextTag = aTag;
}
/**
 * <HSBC>
 * set the State out format
 */
public void setInterStateFormatTag(Tag newInterStateFormatTag) {
	interStateFmt = newInterStateFormatTag;
}
/**
 * <HSBC>
 * Set the keep context flag
 */
public void setKeepContext(boolean keep) {
	keepCtxt = keep;
}

/**
 * This method sets the final page of this state. This atribute only makes sense 
 * in final states. Represents the name of the page that will be displayed when
 * the processor reaches the final state.
 *
 * @param newFinalPage java.lang.String
 */
public void setFinalPage(java.lang.String newFinalPage) {
	finalPage = newFinalPage;
}
/**
 * Sets the return type attribute of this object. Return types were introduced
 * for HTML processor states, and are used to process certain state types for
 * which we wish to return control from the normal state machine flow. We however
 * may wish to execute these states after the first time they return control. The
 * return type attribute controls whether or not a state that would normally return
 * control will actually be processed.
 * 
 * @param newIgnoreReturnType boolean
 */
public void setIgnoreReturnType(boolean newIgnoreReturnType) {
	ignoreReturnType = newIgnoreReturnType;
}
/**
 * Sets the input format tag attribute of this class.
 * 
 * @param newInputFormatTag com.ibm.dse.base.Tag
 */
public void setInputFormatTag(com.ibm.dse.base.Tag newInputFormatTag) {
	inputFormatTag = newInputFormatTag;
}
/**
 * Sets the output format tag attribute format.
 * 
 * @param newOutputFormatTag com.ibm.dse.base.Tag
 */
public void setOutputFormatTag(com.ibm.dse.base.Tag newOutputFormatTag) {
	outputFormatTag = newOutputFormatTag;
}
/**
 * Provides a housekeeping process for the DSEState as a notifier instance. This is called either from 
 * the application or from the context when it is destroyed or unchained.  The "runningModeFlag" and 
 * the "done" value are set to indicate that the State is finished, the potential threads this State 
 * could be in are interrupted, and the State unsubscribes from the shared notifiers.
 *
 * Subclasses should overwrite this implementation to perform the actions required to release the resources 
 * acquired by the notifier. 
 */
public void terminate() {
	super.terminate();

	this.inputFormatTag = null;
	this.outputFormatTag = null;
	
    
}
}
