package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2001
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.*;
import com.ibm.dse.base.types.*;
/**
 * This is a special untyped IndexedCollection (i.e. with no descriptor) to validate fields coming from a
 * HTML browser. The infrastructure finds out that certain data elements
 * coming in the request data belong to an IndexedCollection. Then, a Vector
 * of DataFields are passed to this instance of HtmlIndexedCollection in order
 * to validate that the DataFields contain elements that have been defined
 * in the IndexedCollection. The actual object inside the DataFields is a final
 * object inside the data structure defined inside the IndexedCollection. Since
 * this structure may be complex, it may contain nested KeyedCollections with
 * as many DataElements inside. This means that the DataFields that the infrastructure
 * is trying to validate will contain as name the full qualified path name,
 * so this HtmlIndexedCollection may search for the actual final DataField
 * in the created DataElement inside the IndexedCollection.
 *
 * @copyright(c) Copyright IBM Corporation 2001
 */
public class HtmlIndexedCollection extends com.ibm.dse.base.IndexedCollection {
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2001 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
/**
 * HtmlIndexedCollection constructor
 */
public HtmlIndexedCollection() {
	super();
}
/**
 * HtmlIndexedCollection constructor
 * @param aName java.lang.String
 * @exception java.io.IOException
 */
public HtmlIndexedCollection(String aName) throws java.io.IOException {
	super(aName);
}
/**
 * Update the ErrorInfo attribute with the contents of the Exception
 * @param inputObject Object
 * @param e DSETypeException
 */
public void updateErrorInfo(Object inputObject, DSETypeException e){

	ErrorInfo errorInfo=new ErrorInfo(inputObject);
	errorInfo.put(e.getCode(), e.getMessage());
	// iterate throgh nested exceptions
	DSETypeException ne=e;
	boolean moreErrors=true;
	while (moreErrors) {
		try {
			ne=(DSETypeException)ne.getNestedException();
			errorInfo.put(ne.getCode(), ne.getMessage());
		}
		catch (Exception exc) {
			moreErrors=false;
		}
	}
	setErrorInfo(errorInfo);
}
/**
 * Validates the object passed (that must be an indexed collection or a vector) and if successful, 
 * the object is returned, otherwise an appropriate exception is thrown. 
 * @param toValidate Object
 * @param descriptor PropertyDescription
 * @exception DSETypeException Thrown if the validation is unsuccessful.
 */
public Object validate(Object toValidate) throws DSETypeException{
	int index=0;	
	PropertyDescription descriptor=getDescriptor();
	if (descriptor!=null) {
		return descriptor.validate(toValidate,DSEHtmlProcessor.HTML);
	}
	else
		if (toValidate instanceof Vector) return validate((Vector) toValidate);
		else
			if (toValidate instanceof DataField) return validate((DataField) toValidate);							
			else
				if (toValidate instanceof IndexedCollection) return validate((IndexedCollection) toValidate);

	return super.validate(toValidate);
}
/**
 * Validates the object passed (that must be a DataField) and if successful, 
 * the object is returned, otherwise an appropriate exception is thrown.
 * The DataField contains the name of the final DataField that must be created
 * in the IndexedCollection.
 * The format of the name should be: index+"."+"path to the final DataField"
 * This method will:
 * 		- Create as many elements in the iColl as needed to have an element at the
 		  index passed as the name of the element.
 *		- Let the new created DataField to validate the input value
 *		- If validation succeeds, the field is kept in the IColl
 *		- Otherwise, the ErrorInfo object of the IColl is updated with the
 *		  Exception contents and the DataField is removed (only if it was created)
 * @return java.lang.Object
 * @param toValidate DataField
 * @exception DSETypeException Thrown if the validation is unsuccessful.
 */
public Object validate(DataField toValidate) throws DSETypeException{

	boolean elementHasBeenCreated=false;
	DataElement createdDataElement = null;
	DataField targetDataField = null;

	try
	{	try {targetDataField = (DataField) getElementAt(toValidate.getName());} catch (Exception e) {};
		if (targetDataField == null)
		{
			
			// Add so many elements as needed to reach the index of the incoming element
			int newSize = 1 + ( new Integer( toValidate.getName().substring(0, toValidate.getName().indexOf(com.ibm.dse.base.Constants.dot)) ).intValue() );
			while ( size() < newSize )
			{
				createdDataElement = (DataElement)DataElement.getExternalizer().convertTagToObject(getElementSubTag());
				addElement(createdDataElement);
			}

			elementHasBeenCreated = true;
			targetDataField = (DataField) getElementAt(toValidate.getName());
		}
	} catch (Exception e) {
		try { if (elementHasBeenCreated) removeElementAt(size()-1);} catch (Exception e2) {};
		String msg = "Cannot create a new DataElement for the IColl "+getName()+" based on the Subtag defined";
		throw new DSETypeException(DSETypeException.critical, "Invalid argument", msg, e);}
	
	try {return targetDataField.validate(toValidate.getValue(),DSEHtmlProcessor.HTML);}
	catch (DSETypeException e) 
	{	try { if (elementHasBeenCreated) removeElementAt(size()-1);} catch (Exception e2) {};
		updateErrorInfo(toValidate,e);
		throw e;
	}
}
/**
 * Validates the object passed (that must be an indexed collection) and if successful, 
 * the object is returned, otherwise an appropriate exception is thrown. 
 * @param iColl Object
 * @exception DSETypeException Thrown if the validation is unsuccessful.
 */
public Object validate(IndexedCollection iColl) throws DSETypeException{
	int index=0;	
	try {
		DSETypeException allExceptions = null;
		while (index<iColl.size()) {
			DataElement element = (DataElement)iColl.getElementAt(index);
			// if element is a DataField and not a KeyedCollection then value to validate is contained
			Object innerToValidate = element;
			if (element instanceof DataField) {
				innerToValidate = ((DataField)element).getValue();
			}
			try
			{
				element.validate(innerToValidate,DSEHtmlProcessor.HTML);
			}
			catch (DSETypeException e)
			{
				if (allExceptions == null)
				{
					allExceptions=e;
				}
				else
				{
					allExceptions=e.chainTo(allExceptions);
				}
			}
			index++;
		}
		if (allExceptions!=null) throw allExceptions;
		else return iColl;
	}
	catch (DSEObjectNotFoundException e) { // This sould'nt happen
		String msg = "ObjectNotFoundException trying to validate element at "+index+" in IndexedCollection ";
		throw new DSETypeException(DSETypeException.harmless, "Invalid argument", msg, e);
	}
	catch (ClassCastException e) {
		String msg = "Object "+iColl.getName()+" must be an IndexedCollection or a subclass of it.";
		throw new DSETypeException(DSETypeException.harmless, "Invalid argument", msg, e);
	}

}
/**
 * Validates the set of objects passed (that must be DataField's) and if successful,
 * the set is returned, otherwise an appropriate exception is thrown.
 * The DataField's contain the name of the final DataField that must be created
 * in the IndexedCollection.
 * The format of the name should be: index+"."+"path to the final DataField"
 * This method will:
 * 		- Create the elements inside this IColl, unless they already exist
 *		- Let the new created DataFields to validate the input value
 *		- If validation succeeds, the fields are kept in the IColl
 *		- Otherwise, the ErrorInfo object of the IColl is updated with the
 *		  Exception contents and the DataFields are removed (only if they were created)
 * @return java.lang.Object
 * @param elements Vector
 * @exception DSETypeException Thrown if the validation is unsuccessful.
 */
public Object validate(Vector elements) throws DSETypeException{

	DSETypeException allExceptions = null;
	
	int index=0;
	int vectorSize=elements.size();
	while (index<vectorSize) {
		try
		{
			DataField element = (DataField)elements.elementAt(index);
			validate(element);
		}
		catch (DSETypeException e)
		{
			if (allExceptions == null)
			{
				allExceptions=e;
			}
			else
			{
				allExceptions=e.chainTo(allExceptions);
			}
		}
		index++;
	}
	if (allExceptions!=null)
		{
			updateErrorInfo(elements,allExceptions);
			throw allExceptions;
		}
	else return elements;
}
}
