package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.*;
import com.ibm.dse.base.types.DSETypeException;
/**
 * This class implements a error management toolkit for validators. The
 * base validator class forces the user to manage a string array of error
 * messages that may change in size dynamically as validation is carried out.
 * To alleviate this, this class manages a collection of error messages that
 * can be accessed using the addMessage(String) and deleteMessage(String) methods.
 * At the end of validation, these error messages are composed into a string
 * array and returned automatically. The main caveat of using this class is that
 * subclasses must implement their validation logic in the doXValidate(...) method
 * instead of the xValidate(...) method.
 * 
 * @copyright(c) Copyright IBM Corporation 2000, 2003.
 */
public abstract class HtmlProcessorXValidate implements ProcessorXValidate
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2003 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	private java.util.Vector messages = new java.util.Vector(8);
/**
 * HtmlProcessorXValidate constructor.
 */
public HtmlProcessorXValidate() {
	super();
}
/**
 * Adds a message to the collection of error messages for this
 * validator.
 * 
 * @param message java.lang.String
 */
public void addMessage(String message)
{
	messages.addElement(message) ;
}
/**
 * Removes a message from the collection of messages for this validator.
 * 
 * @param message java.lang.String
 */
public void deleteMessage(String message)
{
	messages.removeElement(message) ;
}
/**
 * Should be overriden by subclasses to do the actual work of semantic 
 * validation or validation based on business rules for the given field. This method 
 * should contain the logic that would normally have been placed in the 
 * validate(String, DataField, Context) method of a class that
 * subclasses from com.ibm.dse.automaton.html.ProcessorXValidate.
 * 
 * @param fullyQualifiedName java.lang.String 
 * @param df com.ibm.dse.base.DataField
 * @param context com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.types.DSETypeException - If the field has not been 
 * successfully validated.
 */
protected void doValidate(String fullyQualifiedName, DataField df, Context ctxt) throws com.ibm.dse.base.types.DSETypeException
{
}
/**
 * Should be overriden by subclasses to do the actual work of
 * cross-validation. This method should contain the logic that would normally
 * have been placed in the xValidate(Context, String) method of a class that
 * subclasses from com.ibm.dse.automaton.html.ProcessorXValidate. Users of this 
 * method can utilize addMessage(String) and deleteMessage(String) to manage
 * error messages. They also do not have to worry about creating and resizing
 * string arrays to hold their error messages. The class handles all of this.
 * 
 * @param context com.ibm.dse.base.Context
 * @param transitionName java.lang.String
 * @exception java.lang.Exception
 */
public abstract void doXValidate(Context context, String transitionName) throws java.lang.Exception;
/**
 * This is equivalent to calling isEmpty(String, true)
 * 
 * @return boolean
 * @param value java.lang.String
 */
public boolean isEmpty(String value)
{
	return isEmpty(value, true) ;
}
/**
 * Returns true if the string passed to it as argument
 * is either null or if it contains no data. If the boolean <i>trim</i>
 * argument is true, the string argument is trimmed before it
 * is validated.
 * 
 * @return boolean
 * @param value java.lang.String
 */
public boolean isEmpty(String value, boolean trim)
{
	if(value == null) return true ;

	if(trim) return value.trim().equals("") ;
	else return value.equals("") ;
}
/**
 * Delegates the actual work of semantic validation for a data field to subclasses. 
 * It then goes through the chain of DSEExceptions to collect the whole set of generated error messages 
 * and update the data field errorInfo accordingly.
 *  
 * @param fullyQualifiedName java.lang.String 
 * @param df com.ibm.dse.base.DataField
 * @param context com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.types.DSETypeException - If the field has not been 
 * successfully validated.
 */
public void validate(String fullyQualifiedName, DataField df, Context ctxt) throws com.ibm.dse.base.types.DSETypeException
{
	try
	{
		doValidate(fullyQualifiedName, df, ctxt);
	}
	catch (DSETypeException e)
	{
		try
		{	DataElement ctxtData=ctxt.getElementAt(fullyQualifiedName);
			ErrorInfo errorInfo=ctxtData.getErrorInfo();
			if (errorInfo==null) 
			{
				errorInfo=new ErrorInfo(df.getValue());
				ctxtData.setErrorInfo(errorInfo);
			}
			else errorInfo.setInputObject(df.getValue());
			errorInfo.put(e.getCode(), e.getMessage());
			// iterate through nested exceptions
			Exception ne=e.getNestedException();
			while (ne!=null) 
			{
				String code=null;
				if (ne instanceof DSETypeException)
				{
					errorInfo.put(((DSETypeException)ne).getCode(), ne.getMessage());
					ne=((DSETypeException)ne).getNestedException();
				}
				else ne=null;
			}
			ctxtData.setErrorInfo(errorInfo);
		} catch (DSEObjectNotFoundException e2) {}
		throw e;
	}
}
/*
 * @note This method is not used for HTML processor validators.
 */
public java.lang.String[] xValidate(Context ctx)
{
	return null;
}
/**
 * Delegates the actual work of cross validation to subclasses. It
 * then collects any generated error messages and returns them as an array of
 * strings as required by the toolkit
 *
 * @return java.lang.String[]
 * @param context com.ibm.dse.base.Context
 * @param transitionName java.lang.String
 * @exception java.lang.Exception
 */

public java.lang.String[] xValidate(Context context, String transitionName) throws Exception
{
	messages.removeAllElements();
	
	doXValidate(context, transitionName) ;

	int count = messages.size() ;
	if(count > 0)
	{
		int i = 0 ;
		String[] errors = new String[count] ;

		for(java.util.Enumeration e = messages.elements() ; e.hasMoreElements() ; i++)
		{
			errors[i] = (String) e.nextElement() ;
		}

		return errors ;
	}

	else return null ;
}
}
